#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_types
# @Time         : 2023/12/19 09:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

completion_keys = get_function_params()


class SpeechCreateRequest(BaseModel):
    input: str
    model: str = 'tts'
    voice: str = "alloy"
    response_format: Literal["mp3", "opus", "aac", "flac"] = "mp3"
    speed: float = 1


# todo: 结构体

chat_completion = {
    "id": "chatcmpl-id",
    "object": "chat.completion",
    "created": 0,
    "model": "LLM",
    "choices": [
        {
            "message": {"role": "assistant", "content": ''},
            "index": 0,
            "finish_reason": "stop",
            "logprobs": None
        }
    ],
    "usage": {"completion_tokens": 0, "prompt_tokens": 0, "total_tokens": 0}

}

chat_completion_chunk = {
    "id": "chatcmpl-id",
    "object": "chat.completion.chunk",
    "created": 0,
    "model": "LLM",
    "choices": [
        {
            "delta": {"role": "assistant", "content": ''},
            "index": 0,
            "finish_reason": None,
            "logprobs": None
        }
    ]
}

# 通用
chat_completion = ChatCompletion.model_validate(chat_completion)
chat_completion_chunk = ChatCompletionChunk.model_validate(chat_completion_chunk)
chat_completion_chunk_stop = chat_completion_chunk.model_copy(deep=True)
chat_completion_chunk_stop.choices[0].finish_reason = "stop"

# ONEAPI_SLOGAN
ONEAPI_SLOGAN = os.getenv("ONEAPI_SLOGAN", "\n\n[永远相信美好的事情即将发生.](https://api.chatllm.vip/)\n\n")

chat_completion_slogan = chat_completion.model_copy(deep=True)
chat_completion_slogan.choices[0].message.content = ONEAPI_SLOGAN

chat_completion_chunk_slogan = chat_completion_chunk.model_copy(deep=True)
chat_completion_chunk_slogan.choices[0].delta.content = ONEAPI_SLOGAN

# ERROR
chat_completion_error = chat_completion.model_copy(deep=True)
chat_completion_chunk_error = chat_completion_chunk.model_copy(deep=True)

# PPU
chat_completion_per = chat_completion.model_copy(deep=True)
chat_completion_per.choices[0].message.content = "按次收费"
chat_completion_per.choices[0].finish_reason = "stop"
chat_completion_chunk_ppu = chat_completion_chunk.model_copy(deep=True)
chat_completion_chunk_ppu.choices[0].delta.content = "按次收费"


class ChatCompletionRequest(BaseModel):
    model: str
    messages: List[Dict[str, str]] = [{'role': 'user', 'content': 'hi'}]
    temperature: Optional[float] = 0
    top_p: Optional[float] = 1.0
    n: Optional[int] = 1
    max_tokens: Optional[int] = None
    stop: Optional[Union[str, List[str]]] = None
    stream: Optional[bool] = False
    presence_penalty: Optional[float] = 0.0
    frequency_penalty: Optional[float] = 0.0
    user: Optional[str] = None

    # extra
    # extra_headers: Headers | None = None,
    # extra_query: Query | None = None,
    extra_body: dict = None,
    # 拓展字段
    # additional_kwargs: Optional[Dict[str, Any]] = Field(default_factory=dict)

    model_config = {
        "json_schema_extra": {
            "examples": [
                {
                    "model": "gpt-3.5-turbo",
                    "messages": [
                        {
                            "role": "user",
                            "content": "hi"
                        }
                    ],
                    "stream": False
                },

                # url
                {
                    "model": "url-gpt-3.5-turbo",
                    "messages": [
                        {
                            "role": "user",
                            "content": "总结一下https://mp.weixin.qq.com/s/Otl45GViytuAYPZw3m7q9w"
                        }
                    ],
                    "stream": False
                }
            ]
        }
    }


class ImageRequest(BaseModel):
    prompt: str
    model: str = 'dall-e-3'
    n: int = 1
    quality: str = 'standard'
    response_format: Literal["url", "b64_json"] = "url"
    size: Literal["256x256", "512x512", "1024x1024", "1792x1024", "1024x1792"] = '1792x1024'  # 测试默认值
    style: Literal["vivid", "natural"] = "vivid"
    extra_body: dict = {}


model_config = {
    "json_schema_extra": {
        "examples": [
            {
                "model": "gpt-3.5-turbo",
                "messages": [
                    {
                        "role": "user",
                        "content": "hi"
                    }
                ],
                "stream": False
            },

            # url
            {
                "model": "url-gpt-3.5-turbo",
                "messages": [
                    {
                        "role": "user",
                        "content": "总结一下https://mp.weixin.qq.com/s/Otl45GViytuAYPZw3m7q9w"
                    }
                ],
                "stream": False
            },

            {
                "model": "chat-dall-e-3",
                "messages": [
                    {
                        "role": "user",
                        "content": "总结一下https://mp.weixin.qq.com/s/Otl45GViytuAYPZw3m7q9w"
                    }
                ],
                "stream": False
            }
        ]
    }
}

if __name__ == '__main__':
    # data = {"stream": True, "model": "gpt-3.5-turbo", "messages": [{"role": "user", "content": "你好"}]}
    # print(ChatCompletionRequest(request=data).request)
    # print(chat_completion_error)
    # print(chat_completion_slogan)

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : stream_response
# @Time         : 2023/5/26 14:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


def generate_response(id, content: str, chat: bool = True):
    # 客户端会更新这些值
    _time = int(time.time())

    if chat:
        return {
            "id": f"chatcmpl-{id}",
            "object": "chat.completion",
            "created": _time,
            "model": "gpt-3.5-turbo-0301",
            "usage": {
                "prompt_tokens": 0,
                "completion_tokens": 0,
                "total_tokens": 0,
            },
            "choices": [
                {
                "message": {"role": "assistant", "content": content},
                "finish_reason": "stop", "index": 0
                }
            ]
        }
    else:
        return {
            "id": f"cmpl-{id}",
            "object": "text_completion",
            "created": _time,
            "model": "text-davinci-003",
            "choices": [
                {
                    "text": content,
                    "index": 0,
                    "logprobs": None,
                    "finish_reason": "stop"
                }
            ],
            "usage": {
                "prompt_tokens": 0,
                "completion_tokens": 0,
                "total_tokens": 0
            }
        }


def generate_stream_response_start(id):
    _time = int(time.time())

    return {
        "id": f"chatcmpl-{id}",
        "object": "chat.completion.chunk",
        "created": _time,
        "model": "gpt-3.5-turbo-0301",
        "choices": [{"delta": {"role": "assistant"}, "index": 0, "finish_reason": None}]
    }


def generate_stream_response(id, content: str, chat: bool = True):
    _time = int(time.time())

    if chat:
        return {
            "id": f"chatcmpl-{id}",  # TODO
            "object": "chat.completion.chunk",
            "created": _time,
            "model": "gpt-3.5-turbo-0301",
            "choices": [{"delta": {"content": content}, "index": 0, "finish_reason": None}]
        }
    else:
        return {
            "id": f"cmpl-{id}",
            "object": "text_completion",
            "created": _time,
            "choices": [
                {
                    "text": content,
                    "index": 0,
                    "logprobs": None,
                    "finish_reason": None,
                }
            ],
            "model": "text-davinci-003"
        }


def generate_stream_response_stop(id, chat: bool = True):
    _time = int(time.time())

    if chat:
        return {
            "id": f"chatcmpl-{id}",
            "object": "chat.completion.chunk",
            "created": _time,
            "model": "gpt-3.5-turbo-0301",
            "choices": [{"delta": {}, "index": 0, "finish_reason": "stop"}]
        }
    else:
        return {
            "id": f"cmpl-{id}",
            "object": "text_completion",
            "created": _time,
            "choices": [
                {"text": "", "index": 0, "logprobs": None, "finish_reason": "stop"}],
            "model": "text-davinci-003",
        }



if __name__ == '__main__':
    print(generate_stream_response(''))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chaturl
# @Time         : 2023/9/5 16:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
"""
"使用WebPilot"（触发开启WebPilot插件功）
"浏览这个URL：[URL]"（触发获取URL的网页内容功能）
"重写这个段落"（触发重写网页内容功）
"翻译这个网页到法语"（触发翻译网页内容功）
"我想要Steve Jobs的语气"（触发设置回答的语气功）
"Bonjour"（触发设置对话语言为法语功）
"提供这个网页的HTML元信息"（触发获取网页的HTML元信息功）
"提供这个网页的链接"（触发获取网页中的链接功）
"我想要一个简洁的摘要"（触发获取网页内容的简洁摘要功）
"我有一个问题：[问题]"（触发根据网页内容回答问题功）

"""

from meutils.pipe import *
from meutils.str_utils.regular_expression import parse_url
from meutils.decorators.retry import retrying

from openai import OpenAI, AsyncOpenAI
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.schemas.openai_types import chat_completion_chunk, completion_keys


class Chat(object):

    def __init__(self, api_key=None, **kwargs):
        self.client = OpenAI(
            api_key=api_key,
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1"),
            http_client=httpx.Client(follow_redirects=True)
        )

    @retrying
    def create(self, request: ChatCompletionRequest):
        # 提示词扩充
        prompt = """
        你是个网页问答助手，擅长根据网页内容回答问题，忽略url链接，我会给你具体的网页内容用```包裹.
        """.strip()

        if request.messages[0]['role'] == 'system':
            request.messages[0]['content'] += prompt
        else:
            request.messages = [{'role': 'system', 'content': prompt}] + request.messages

        for message in request.messages[1:]:
            message['content'] = self.content2url_content(message['content'])

        logger.debug(request.messages)

        # 通用化 to_openai_completion_params，多余的放在 extra_body 里
        data = request.model_dump()
        data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key

        return self.client.chat.completions.create(**data)

    def create_sse(self, request: ChatCompletionRequest):
        response = self.create(request)
        if request.stream:
            from sse_starlette import EventSourceResponse
            generator = (chunk.model_dump_json() for chunk in response)
            return EventSourceResponse(generator, ping=10000)
        return response

    @lru_cache
    def content2url_content(self, content):
        urls = parse_url(content)

        for i, url in enumerate(urls):
            response = httpx.get(
                f"""{os.getenv("SPIDER_URL")}/preview/spider/playwright""",
                params={"url": url}
            ).json()
            url_content = response.get('content').strip()
            content = content.replace(url, f"```url\n{url_content}```")
            # content = content.replace(url, f"```\n{response}```")

        return content

    async def get_context(self, urls, params, **kwargs):
        async with httpx.AsyncClient() as client:
            responses = (await client.get(url, params=params, **kwargs) for url in urls)
            return [response.text for response in responses]


if __name__ == '__main__':
    url = "https://mp.weixin.qq.com/s/xxzNx6nPJTfovCzOBXTWKA"

    # print(Chat().url2content(url))

    data = {
        'model': 'gpt-3.5-turbo',
        # 'model': 'gpt-4-turbo',

        'messages': [
            {'role': 'user', 'content': f'有哪些名词  {url}'},
        ],
        'stream': False
    }

    print(Chat().create(ChatCompletionRequest(**data)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ppu_utils
# @Time         : 2024/1/8 16:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 按次计费

from meutils.pipe import *
from openai import OpenAI


# @background_task
def per_create(api_key: Optional[str] = None, base_url: Optional[str] = None, model: str = 'per'):
    openai = OpenAI(
        api_key=api_key,
        base_url=base_url or "https://api.chatllm.vip/v1",
        http_client=httpx.Client(follow_redirects=True)
    )
    return openai.chat.completions.create(model=model, messages=[{"role": "user", "content": "hi"}])


if __name__ == '__main__':
    print(per_create())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatglm
# @Time         : 2023/9/26 15:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators.retry import retrying

from chatllm.schemas.openai_api_protocol import *


class ChatGLMCompletion(object):
    @classmethod
    def create(
        cls,
        messages: List[Dict[str, Any]],  # [{'role': 'user', 'content': '讲个故事'}]
        **kwargs,
    ):
        try_import('zhipuai')
        import zhipuai
        from zhipuai.model_api import api

        zhipuai.api_key = kwargs.pop('api_key')
        api.post = retrying(api.post, predicate=lambda x: x is None)
        api.stream = retrying(api.stream, predicate=lambda x: x is None)

        if kwargs.get('stream'):
            return cls._stream_create(messages, **kwargs)
        else:
            return cls._create(messages, **kwargs)

    @staticmethod
    def _create(messages, **kwargs):
        try_import('zhipuai')
        import zhipuai
        data = zhipuai.model_api.invoke(prompt=messages, **kwargs).get('data', {})
        choices = data.get('choices', [])
        if choices:
            choice = ChatCompletionResponseChoice(index=0, message=ChatMessage(**choices[0]), finish_reason='stop')
            _ = ChatCompletionResponse(
                model=kwargs.get('model', 'chatglm_lite'),
                choices=[choice],
                usage=UsageInfo(**data.pop('usage', {}))
            ).dict()
            return _
        return {}

    @staticmethod
    def _stream_create(messages, **kwargs):

        try_import('zhipuai')
        import zhipuai

        resp = zhipuai.model_api.sse_invoke(prompt=messages, **kwargs).events()

        finish_reason = None
        usage = UsageInfo()
        for event in resp:
            if event.event == 'finish':
                finish_reason = 'stop'
                usage = json.loads(event.meta).get('usage', {})
                usage = UsageInfo(**usage)

            delta = DeltaMessage(role='assistant', content=event.data)
            choice = ChatCompletionResponseStreamChoice(
                index=0,
                delta=delta,
                finish_reason=finish_reason,
            )
            stream_resp = ChatCompletionStreamResponse(
                choices=[choice],
                model=kwargs.get('model', 'chatglm_lite'),
                usage=usage
            ).dict()

            yield stream_resp


if __name__ == '__main__':
    from meutils.pipe import *

    r = ChatGLMCompletion.create(
        messages=[{'role': 'user', 'content': '1+1'}],
        stream=False,
        api_key=os.getenv('CHATGLM_API_KEY'),
        model='chatglm_lite'
    )
    print(r)
    for i in r:
        print(i)



from langchain.docstore.document import Document
import re


def under_non_alpha_ratio(text: str, threshold: float = 0.5):
    """Checks if the proportion of non-alpha characters in the text snippet exceeds a given
    threshold. This helps prevent text like "-----------BREAK---------" from being tagged
    as a title or narrative text. The ratio does not count spaces.

    Parameters
    ----------
    text
        The input string to test
    threshold
        If the proportion of non-alpha characters exceeds this threshold, the function
        returns False
    """
    if len(text) == 0:
        return False

    alpha_count = len([char for char in text if char.strip() and char.isalpha()])
    total_count = len([char for char in text if char.strip()])
    try:
        ratio = alpha_count / total_count
        return ratio < threshold
    except:
        return False


def is_possible_title(
        text: str,
        title_max_word_length: int = 20,
        non_alpha_threshold: float = 0.5,
) -> bool:
    """Checks to see if the text passes all of the checks for a valid title.

    Parameters
    ----------
    text
        The input text to check
    title_max_word_length
        The maximum number of words a title can contain
    non_alpha_threshold
        The minimum number of alpha characters the text needs to be considered a title
    """

    # 文本长度为0的话，肯定不是title
    if len(text) == 0:
        print("Not a title. Text is empty.")
        return False

    # 文本中有标点符号，就不是title
    ENDS_IN_PUNCT_PATTERN = r"[^\w\s]\Z"
    ENDS_IN_PUNCT_RE = re.compile(ENDS_IN_PUNCT_PATTERN)
    if ENDS_IN_PUNCT_RE.search(text) is not None:
        return False

    # 文本长度不能超过设定值，默认20
    # NOTE(robinson) - splitting on spaces here instead of word tokenizing because it
    # is less expensive and actual tokenization doesn't add much value for the length check
    if len(text) > title_max_word_length:
        return False

    # 文本中数字的占比不能太高，否则不是title
    if under_non_alpha_ratio(text, threshold=non_alpha_threshold):
        return False

    # NOTE(robinson) - Prevent flagging salutations like "To My Dearest Friends," as titles
    if text.endswith((",", ".", "，", "。")):
        return False

    if text.isnumeric():
        print(f"Not a title. Text is all numeric:\n\n{text}")  # type: ignore
        return False

    # 开头的字符内应该有数字，默认5个字符内
    if len(text) < 5:
        text_5 = text
    else:
        text_5 = text[:5]
    alpha_in_text_5 = sum(list(map(lambda x: x.isnumeric(), list(text_5))))
    if not alpha_in_text_5:
        return False

    return True


def zh_title_enhance(docs: Document) -> Document:
    title = None
    if len(docs) > 0:
        for doc in docs:
            if is_possible_title(doc.page_content):
                doc.metadata['category'] = 'cn_Title'
                title = doc.page_content
            elif title:
                doc.page_content = f"下文与({title})有关。{doc.page_content}"
        return docs
    else:
        print("文件不存在")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2023/4/20 12:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import copy

import torch
from transformers import AutoTokenizer, AutoModel

from meutils.pipe import *
from chatllm.utils.gpu_utils import load_chatglm_on_gpus
from chatllm.schemas.openai_types import chat_completion_chunk, completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest

DEVICE = (
    os.getenv('DEVICE') if 'DEVICE' in os.environ
    else "cuda" if torch.cuda.is_available()
    else "mps" if torch.backends.mps.is_available()
    else "cpu"
)

xgroup = Pipe(lambda ls, step=3, overlap_rate=0: [ls[max(idx - int(step * overlap_rate), 0): idx + step] for idx in
                                                  range(0, len(ls), step)])


def textsplitter(text, chunk_size=512, overlap_rate=0.2, sep=''):  # 简单粗暴
    return text.lower().split() | xjoin(sep) | xgroup(chunk_size, overlap_rate)


def load_llm(model_name_or_path="THUDM/chatglm-6b", device=DEVICE, num_gpus=2, **kwargs):
    model = AutoModel.from_pretrained(model_name_or_path, trust_remote_code=True)
    tokenizer = AutoTokenizer.from_pretrained(model_name_or_path, trust_remote_code=True)

    if torch.cuda.is_available() and device.lower().startswith("cuda"):
        num_gpus = min(num_gpus, torch.cuda.device_count())

        if num_gpus == 1:  # 单卡
            model = model.half().cuda()
            # model.transformer.prefix_encoder.float()
        elif 'chatglm' in model_name_or_path:  # chatglm多卡
            model = load_chatglm_on_gpus(model_name_or_path, num_gpus)
            logger.info('多卡加载模型')

    else:
        model = model.float().to(device)

    return model.eval(), tokenizer


def load_llm4chat(model_name_or_path="THUDM/chatglm-6b", device=DEVICE, num_gpus=2, stream=True, **kwargs):
    model, tokenizer = load_llm(model_name_or_path, device, num_gpus, **kwargs)
    if stream and hasattr(model, 'stream_chat'):
        return partial(model.stream_chat, tokenizer=tokenizer)  # 可以在每一次生成清GPU
    else:
        return partial(model.chat, tokenizer=tokenizer)


def to_openai_completion_params(request: Union[dict, ChatCompletionRequest]):
    data = copy.deepcopy(request)
    if isinstance(request, ChatCompletionRequest):
        data = request.model_dump()

    extra_body = {}
    for key in list(data):
        if key not in completion_keys:
            extra_body.setdefault(key, data.pop(key))

    data['extra_body'] = extra_body  # 拓展字段

    return data


# data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key

if __name__ == '__main__':
    # model, tokenizer = load_llm("/CHAT_MODEL/chatglm", device='cpu')
    print(to_openai_completion_params(ChatCompletionRequest(model='gpt')))
    # print(completion_keys)



from enum import IntEnum
import os

REPO_PATH = os.path.dirname(os.path.dirname(__file__))

##### For the gradio web server
SERVER_ERROR_MSG = (
    "**NETWORK ERROR DUE TO HIGH TRAFFIC. PLEASE REGENERATE OR REFRESH THIS PAGE.**"
)
MODERATION_MSG = "YOUR INPUT VIOLATES OUR CONTENT MODERATION GUIDELINES. PLEASE FIX YOUR INPUT AND TRY AGAIN."
CONVERSATION_LIMIT_MSG = "YOU HAVE REACHED THE CONVERSATION LENGTH LIMIT. PLEASE CLEAR HISTORY AND START A NEW CONVERSATION."
INACTIVE_MSG = "THIS SESSION HAS BEEN INACTIVE FOR TOO LONG. PLEASE REFRESH THIS PAGE."
# Maximum input length
INPUT_CHAR_LEN_LIMIT = int(os.getenv("FASTCHAT_INPUT_CHAR_LEN_LIMIT", 2560))
# Maximum conversation turns
CONVERSATION_TURN_LIMIT = 50
# Session expiration time
SESSION_EXPIRATION_TIME = 3600
# The output dir of log files
LOGDIR = "."


##### For the controller and workers (could be overwritten through ENV variables.)
CONTROLLER_HEART_BEAT_EXPIRATION = int(
    os.getenv("FASTCHAT_CONTROLLER_HEART_BEAT_EXPIRATION", 90)
)
WORKER_HEART_BEAT_INTERVAL = int(os.getenv("FASTCHAT_WORKER_HEART_BEAT_INTERVAL", 45))
WORKER_API_TIMEOUT = int(os.getenv("FASTCHAT_WORKER_API_TIMEOUT", 100))
WORKER_API_EMBEDDING_BATCH_SIZE = int(
    os.getenv("FASTCHAT_WORKER_API_EMBEDDING_BATCH_SIZE", 4)
)


class ErrorCode(IntEnum):
    """
    https://platform.openai.com/docs/guides/error-codes/api-errors
    """

    VALIDATION_TYPE_ERROR = 40001

    INVALID_AUTH_KEY = 40101
    INCORRECT_AUTH_KEY = 40102
    NO_PERMISSION = 40103

    INVALID_MODEL = 40301
    PARAM_OUT_OF_RANGE = 40302
    CONTEXT_OVERFLOW = 40303

    RATE_LIMIT = 42901
    QUOTA_EXCEEDED = 42902
    ENGINE_OVERLOADED = 42903

    INTERNAL_ERROR = 50001
    CUDA_OUT_OF_MEMORY = 50002
    GRADIO_REQUEST_ERROR = 50003
    GRADIO_STREAM_UNKNOWN_ERROR = 50004
    CONTROLLER_NO_WORKER = 50005
    CONTROLLER_WORKER_TIMEOUT = 50006



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : base
# @Time         : 2023/8/9 15:04
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from langchain.chains.question_answering import load_qa_chain

from meutils.pipe import *

from langchain.chat_models import ChatOpenAI
from langchain.prompts.chat import ChatPromptTemplate, ChatPromptValue
from langchain.schema.language_model import BaseLanguageModel

from chatllm.llmchain.decorators import llm_stream, llm_astream
from chatllm.llmchain.prompts.rag import template, strict_template


class ChatBase(object):

    def __init__(
        self,
        llm: Optional[BaseLanguageModel] = None,
        get_api_key: Optional[Callable[[int], List[str]]] = None,  # 队列
        **kwargs
    ):
        self.llm = llm or ChatOpenAI(model="gpt-3.5-turbo-0613", temperature=0, streaming=True)

        if get_api_key:
            self.llm.openai_api_key = get_api_key(1)[0]

    def chat(self, prompt, **kwargs):
        if isinstance(prompt, ChatPromptValue):
            prompt = prompt.to_string().strip()

        self.switch_model_hook(prompt)

        yield from llm_stream(self.llm.predict)(prompt)

    def achat(self, prompt, **kwargs):
        close_event_loop()
        yield from async2sync_generator(llm_astream(self.llm.apredict)(prompt))

    async def _achat(self, prompt, **kwargs):
        await llm_astream(self.llm.apredict)(prompt)

    def switch_model_hook(self, prompt):  # todo: 可以更丰富些
        """根据tokens切换模型"""
        logger.debug(prompt)
        num_tokens = self.llm.get_num_tokens(prompt)
        if num_tokens < 3600:  # 切换阈值
            self.llm.model_name = 'gpt-3.5-turbo-0613'
        else:
            self.llm.model_name = 'gpt-3.5-turbo-16k-0613'


if __name__ == '__main__':
    cb = ChatBase()
    print(cb.llm.model_name)

    # cb.chat('p') | xprint(end='\n')
    # cb.chat('p' * 100) | xprint(end='\n')

    # ChatBase().chat('1+1') | xprint(end='\n')
    # ChatBase().achat('周杰伦是谁') | xprint(end='\n')

    p = strict_template.format_prompt(context='小明在南京', question='介绍小明')


    # p.to_string()
    # ChatBase().chat(p.to_string()) | xprint(end='\n')

    for i in cb.chat(p):
        print(i, end='')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : glm
# @Time         : 2023/7/24 13:47
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://maas.aminer.cn/dev/api#chatglm_pro

from pydantic import model_validator
from langchain.chat_models.openai import ChatOpenAI
from langchain.utils import get_from_dict_or_env

from langchain.adapters.openai import convert_message_to_dict
from langchain.schema.messages import BaseMessage

from meutils.pipe import *
from chatllm.llmchain.completions import HunYuanCompletion


class HuanYuan(ChatOpenAI):
    """
        chatglm_lite
        chatglm_std
        chatglm_pro
    """
    client: Any  #: :meta private:
    model_name: str = Field(default="hunyuan", alias="model")
    openai_api_key: Optional[str] = Field(default=None, alias="hunyuan_api_key")

    class Config:
        """Configuration for this pydantic object."""

        allow_population_by_field_name = True

    @model_validator(mode='after')
    def validate_environment(cls, values: Dict) -> Dict:
        """Validate that api key and python package exists in environment."""
        # 覆盖 openai_api_key
        values["openai_api_key"] = get_from_dict_or_env(
            values, "hunyuan_api_key", "HUNYUAN_API_KEY"
        )

        values["client"] = HunYuanCompletion

        if values["n"] < 1:
            raise ValueError("n must be at least 1.")
        if values["n"] > 1 and values["streaming"]:
            raise ValueError("n must be 1 when streaming.")
        return values

    @property
    def _llm_type(self) -> str:
        return Path(__file__).name  # 'ernie'

    def get_num_tokens_from_messages(self, messages: List[BaseMessage]) -> int:
        """Calculate num tokens with tiktoken package.

        Official documentation: https://github.com/openai/openai-cookbook/blob/
        main/examples/How_to_format_inputs_to_ChatGPT_models.ipynb"""
        if sys.version_info[1] <= 7:
            return super().get_num_tokens_from_messages(messages)
        model, encoding = self._get_encoding_model()
        tokens_per_message = 3
        tokens_per_name = 1
        num_tokens = 0
        messages_dict = [convert_message_to_dict(m) for m in messages]
        for message in messages_dict:
            num_tokens += tokens_per_message
            for key, value in message.items():
                # Cast str(value) in case the message value is not a string
                # This occurs with function messages
                num_tokens += len(encoding.encode(str(value)))
                if key == "name":
                    num_tokens += tokens_per_name
        # every reply is primed with <im_start>assistant
        num_tokens += 3
        return num_tokens


if __name__ == '__main__':
    from meutils.pipe import *
    from chatllm.llmchain.llms import ChatGLM

    from langchain.chains import LLMChain
    from langchain.prompts import ChatPromptTemplate

    first_prompt = ChatPromptTemplate.from_template("{q}")

    llm = HuanYuan(streaming=True)
    print(llm.openai_api_key)
    c = LLMChain(llm=llm, prompt=first_prompt)

    for i in c.run('你是谁'):
        print(i, end='')

    # from chatllm.llmchain.decorators import llm_stream
    #
    # for i in llm_stream(c.run)('你是谁'):
    #     print(i, end='')



#!/usr/bin/env python

"""Tests for `llm4gpt` package."""

from langchain.agents.agent_toolkits import OpenAPIToolkit

from langchain.callbacks.streaming_stdout_final_only import (
    FinalStreamingStdOutCallbackHandler,
)
